# encoding: utf-8 require "logstash/outputs/base"
require "logstash/namespace"
require "uri"
require "net/http"
require "net/https"
require "zlib"
require 'time'
require 'json'

# 
# This is most useful so you can use logstash to parse and structure
# your logs and ship structured, json events to ThetaPoint.
class LogStash::Outputs::ThetaPoint < LogStash::Outputs::Base
  config_name "thetapoint"

  # The hostname to send logs to. 
  config :host, :validate => :string, :default => "api.theta-point.com"
  config :port, :validate => :number, :default => 443
  config :path, :validate => :string, :default => "bulk"

  # The thetapoint http input key to send to.
  #     https://thetapoint03.theta-point.com/inputs/abcdef12-3456-7890-abcd-ef0123456789
  #                                           ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
  #                                           \---------->   key   <-------------/
  config :key, :validate => :string, :required => true

  # Should the log action be sent over https instead of plain http
  config :proto, :validate => :string, :default => "https"

  # Proxy Host
  config :proxy_host, :validate => :string, :default => nil

  # Proxy Port
  config :proxy_port, :validate => :number, :default => nil

  # Proxy Username
  config :proxy_user, :validate => :string, :default => nil

  # Proxy Password
  config :proxy_password, :validate => :password, :default => nil

  # If true, we send an HTTP POST request every "batch_events" events or
  # "batch_timeout" seconds (whichever comes first).
  config :batch, :validate => :boolean, :default => false, :deprecated => true
  config :batch_events, :validate => :number, :default => 100, :deprecated => true
  config :batch_timeout, :validate => :number, :default => 5, :deprecated => true

  config :compress, :validate => :boolean, :default => true

  public
  def register
    @logger.debug "Initializing ThetaPoint Output", @config
  end

  public
  def receive(event)
    @logger.debug("receive(#{event})")

    send_data(event.to_json)
  end # def receive

  def multi_receive(events)
    @logger.debug("multi_receive(#{events.length})")

    send_data(events.to_json)
  end # def multi_receive


  def send_data(data)
    @logger.debug("send_data(#{data.length})")

    uri = "#{@proto}://#{@host}"
    post_data = data

    # Comress data
    if @compress
      post_data = Zlib::Deflate.deflate(data, Zlib::BEST_COMPRESSION)
      uri << "/zbulk/#{@key}"
    else
      uri << "/#{@key}"
    end

    @logger.debug("URI: #{uri}")
    url = URI.parse(uri)

    http = Net::HTTP::Proxy(@proxy_host,
      @proxy_port,
      @proxy_user,
      @proxy_password.value).new(url.host, url.port)

    if url.scheme == 'https'
      http.use_ssl = true
      http.verify_mode = OpenSSL::SSL::VERIFY_NONE
    end

    request = Net::HTTP::Post.new(url.path)
    request.body = post_data

    begin
      response = http.request(request)
      @logger.debug "Response: #{response}"

      if response.is_a?(Net::HTTPSuccess)
        @logger.debug("Event send to ThetaPoint OK!")
      else
        @logger.warn("HTTP error", :error => response.error!)
      end

    rescue StandardError => e
        @logger.error("An unexpected error occurred", :exception => e.class.name, :error => e.to_s, :backtrace => e.backtrace)
    end # rescue

  end # def send_data

end # class LogStash::Outputs::ThetaPoint
